import datetime as dt

import numpy as np
import pkg_resources

from .globals import magcosRunDir

class MAGCOSmacroFileGenerator():

    _MacroFileTemplateLocation = pkg_resources.resource_filename(__name__,"magcos_running_scripts/AsymptoticDirection.g4mac")

    def __init__(self):
        with open(self._MacroFileTemplateLocation,"r") as templateFile:
            macroFileTemplateString = templateFile.read()
        macroFileSplit = macroFileTemplateString.split("REPLACE_THIS_LINE_WITH_FULL_LOOP_COMMANDS")
        self._MacroFilePrefix = macroFileSplit[0]
        self._MacroFileSuffix = macroFileSplit[1]

    def setZenithList(self,zenithList):
        self._zenithList = zenithList

    def setAzimuthList(self,azimuthList):
        self._azimuthList = azimuthList

    def setLatitudeList(self,latitudeList):
        self._latitudeList = latitudeList

    def setLongitudeList(self,longitudeList):
        self._longitudeList = longitudeList

    def setIopt(self, Iopt):
        self._Iopt = Iopt

    def setDatetime(self, dateAndTime:dt.datetime):
        self._dateAndTime = dateAndTime

    def setRigidityVectorString(self,maxValue:float,minValue:float,nIncrements:int):

        increment = (minValue - maxValue)/(nIncrements - 1)

        self._rigidityVectorString = "\n/MAGCOS/RIGIDITYVECTOR/Reset\n/MAGCOS/RIGIDITYVECTOR/AddValues " + \
                                     str(maxValue) + " " + \
                                     str(increment) + " " + \
                                     str(nIncrements-1) + "\n"

    def setMAGNETOCOSMICSdatetimeParameters(self,dateAndTime:dt.datetime,KpIndex:int):
        self.setDatetime(dateAndTime)
        self.setIopt(KpIndex+1)

    def setMishevMAGNETOCOSMICSParameters(self):
        self.setDefaultMAGNETOCOSMICSrunParameters()
        self.setMAGNETOCOSMICSdatetimeParameters(self._dateAndTime,
                                                KpIndex=4)
    
    # def setDefaultMAGNETOCOSMICSrunParameters(self,maxValue,minValue,nIncrements):
    #     self.setZenithList([0.0])
    #     self.setAzimuthList([0.0])
    #     self.setLatitudeList(np.linspace(-90.0,90.0,37))
    #     self.setLongitudeList(np.linspace(0.0,355.0,72))
    #     self.setRigidityVectorString(maxValue=maxValue,minValue=minValue,nIncrements=nIncrements)

    def setTestMAGNETOCOSMICSrunParameters(self):
        self.setDefaultMAGNETOCOSMICSrunParameters()
        self.setMAGNETOCOSMICSdatetimeParameters(self._dateAndTime,
                                                KpIndex=6)

    def getSingleRunString(self,zenith,azimuth,latitude,longitude):

        runSetupString = "\n/MAGCOS/SOURCE/SetPosition GEOID 100. km " + \
                                    str(latitude) + " " + \
                                    str(longitude) + \
                                    " degree\n/MAGCOS/SOURCE/SetDirection GEOID " + \
                                    str(zenith) + " " + \
                                    str(azimuth) + \
                                    " degree\n"
                                    
        runString = "/MAGCOS/SCENARIO/ComputeAsymptoticDirections AsymptoticDirectionDay_POS_" + \
                        str(latitude) + "_" + \
                        str(longitude) + \
                        "_MOM_" + \
                        str(zenith) + "_" + \
                        str(azimuth) + ".out"

        return runSetupString + runString

    def getMagnetosphericConditionsString(self):

        IoptString = f"/MAGCOS/BFIELD/SetIopt {self._Iopt}"
        datetimeString = f"/MAGCOS/BFIELD/SetStartDate {self._dateAndTime.year} {self._dateAndTime.month} {self._dateAndTime.day} {self._dateAndTime.hour} {self._dateAndTime.minute} {self._dateAndTime.second}"
        magFieldSpecificationString = "\n" + IoptString + "\n" + datetimeString + "\n"
        return magFieldSpecificationString

    def generateMacro(self, lat_and_long_list, zenith_and_azimuth_list):

        magFieldSpecificationString = self.getMagnetosphericConditionsString()

        fullRunningString = ""

        for zenith, azimuth in zenith_and_azimuth_list:
            for latitude, longitude in lat_and_long_list:
                fullRunningString += self.getSingleRunString(zenith, azimuth, latitude, longitude)
        
        fullMacroString = self._MacroFilePrefix + magFieldSpecificationString + self._rigidityVectorString + fullRunningString + self._MacroFileSuffix

        with open(magcosRunDir + "/magcos_running_scripts/runningAsymptoticDirection.g4mac","w") as outputMacroFile:
            outputMacroFile.write(fullMacroString)



from distutils import dir_util
import os
from pathlib import Path
import glob as gb
import shutil
import subprocess

from joblib import Memory

from .MAGCOSmacroFileGenerator import MAGCOSmacroFileGenerator
from .MAGNETOCOSMICSimportingTools import readLotsOfOutputAngleFilesChunked

from .globals import _magnetocosmicsShellScriptPath, magcosRunDir, originalDirectory

def get_magcos_cache_function():
    MAGCOScachedir = f"{os.getcwd()}/cachedMagnetocosmicsRunData"
    MAGCOSmemory = Memory(MAGCOScachedir, verbose=0)

    @MAGCOSmemory.cache()
    def run_or_get_MAGNETOCOSMICS_cache(*args, **kwargs):

        the_MAGCOS_run = MAGNETOCOSMICSrun(*args, **kwargs)

        return the_MAGCOS_run

    return run_or_get_MAGNETOCOSMICS_cache

class MAGNETOCOSMICSrun():

    def __init__(self, 
                array_of_lats_and_longs, array_of_zeniths_and_azimuths, 
                date_and_time,KpIndex,
                maxRigValue,minRigValue,nIncrements):

        Path(f"{magcosRunDir}/magcos_running_scripts/outputFiles/").mkdir(parents=True, exist_ok=True)
        Path(f"{magcosRunDir}/data/").mkdir(parents=True, exist_ok=True)
        Path(f"{magcosRunDir}/bin/").mkdir(parents=True, exist_ok=True)

        os.chdir(f"{magcosRunDir}/magcos_running_scripts/")

        for previousOutputFile in gb.glob(magcosRunDir + "/magcos_running_scripts/outputFiles/*.out"):
            os.remove(previousOutputFile)

        macroFileGenerator = MAGCOSmacroFileGenerator()
        macroFileGenerator.setMAGNETOCOSMICSdatetimeParameters(date_and_time,KpIndex=KpIndex)
        macroFileGenerator.setRigidityVectorString(maxRigValue, minRigValue, nIncrements)

        macroFileGenerator.generateMacro(array_of_lats_and_longs, array_of_zeniths_and_azimuths)

        try:
            magnetocosmics_subprocess_run = subprocess.run(_magnetocosmicsShellScriptPath)
            assert magnetocosmics_subprocess_run.returncode == 0
        except:
            raise Exception("ERROR: running magnetocosmics and processing output files failed, please check the above error messages to debug what might have gone wrong.")

        os.chdir(originalDirectory)

        self.full_rigidity_DF = self.importMAGCOSoutputFiles()

        shutil.rmtree(magcosRunDir)

    def importMAGCOSoutputFiles(self):
        listOfAllFilesToImport = gb.glob(f"{magcosRunDir}/magcos_running_scripts/outputFiles/*.out")
        print("Importing all Magnetocosmics output files to a dataframe...")
        fullRigidityDFGEO = readLotsOfOutputAngleFilesChunked(listOfAllFilesToImport,nChunks=100)
        print("Magnetocosmics importing completed.")

        return fullRigidityDFGEO

    def get_full_rigidity_DF(self):
        return self.full_rigidity_DF


import numpy as np
import pandas as pd
import datetime as dt
import shutil

try:
    assert not (shutil.which("magnetocosmics") is None)
except AssertionError:
    print("ERROR: AsympDirsCalculator currently requires magnetocosmics to be installed, such that the command 'magnetocosmics' can be run in terminal. All AsympDirsCalculator runs will currently fail until magnetocosmics is installed. If you have installed a copy of magnetocosmics, please check to see ensure that your paths have been set correctly, and that magneto-env.sh has been sourced correctly. If you are having issues or suspect there might be a bug, please feel free to report it at https://github.com/ssc-maire/AsymptoticDirectionsCalculator-public/issues , and I will try to respond to your issue as soon as I'm able to.")

from .MAGNETOCOSMICSrunManager import MAGNETOCOSMICSrunManager

def get_magcos_asymp_dirs(
                        array_of_lats_and_longs = np.array(np.meshgrid(np.linspace(-90.0,90.0,37), 
                                                np.linspace(0.0,355.0,72))).T.reshape(-1, 2),
                        array_of_zeniths_and_azimuths = np.array([[0.0, 0.0]]),
                        KpIndex=2,
                        dateAndTime=dt.datetime.now(), #dt.datetime(year=2010,month=6,day=1,hour=0,minute=0,second=0), 
                        highestMaxRigValue = 1010,
                        maxRigValue = 20,
                        minRigValue = 0.1,
                        nIncrements_high = 60,
                        nIncrements_low = 200,
                        cache=False,
                        full_output=False,
                        )->pd.DataFrame:

    print("running MAGNETOCOSMICS to acquire asymptotic directions...")
    MAGCOSrunManager = MAGNETOCOSMICSrunManager(dateAndTime, KpIndex, cache=cache)

    MAGCOSrunManager.set_rigidity_run_params(highestMaxRigValue,maxRigValue,minRigValue,nIncrements_high,nIncrements_low)
    MAGCOSrunManager.set_particle_direction_params(array_of_zeniths_and_azimuths)
    MAGCOSrunManager.run(array_of_lats_and_longs)
    dfOfAllAsymptoticDirections = MAGCOSrunManager.getOutputRigidities().sort_values(["initialLatitude","initialLongitude"])

    if full_output == False:
        dfOfAllAsymptoticDirections = dfOfAllAsymptoticDirections[["initialLatitude","initialLongitude","Rigidity","Lat","Long","Filter"]]

    return dfOfAllAsymptoticDirections

def get_magcos_vcutoffs(
                        array_of_lats_and_longs = np.array(np.meshgrid(np.linspace(-90.0,90.0,37), 
                                                np.linspace(0.0,355.0,72))).T.reshape(-1, 2),
                        array_of_zeniths_and_azimuths = np.array([[0.0, 0.0]]),
                        KpIndex=2,
                        dateAndTime=dt.datetime.now(), #dt.datetime(year=2010,month=6,day=1,hour=0,minute=0,second=0), 
                        highestMaxRigValue = 1010,
                        maxRigValue = 20,
                        minRigValue = 0.1,
                        nIncrements_high = 60,
                        nIncrements_low = 200,
                        cache=False,
                        )->pd.DataFrame:

    full_mag_cos_asymp_dir_DF = get_magcos_asymp_dirs(
                        array_of_lats_and_longs,
                        array_of_zeniths_and_azimuths,
                        KpIndex,
                        dateAndTime,
                        highestMaxRigValue,
                        maxRigValue,
                        minRigValue,
                        nIncrements_high,
                        nIncrements_low,
                        cache,
                        full_output=True,
                        )

    full_vcutoffs_DF = full_mag_cos_asymp_dir_DF.groupby(["initialLatitude","initialLongitude"]).last()[["Rlower","Reffective","Rupper"]]

    return full_vcutoffs_DF




import os
import pkg_resources

global _magnetocosmicsShellScriptPath, \
       originalDirectory, \
       magcosRunDir

_magnetocosmicsShellScriptPath = pkg_resources.resource_filename(__name__,"magcos_running_scripts/runNoRewriteMAGCOSsimulation.sh")

originalDirectory = os.getcwd()

def getUniqueLabelForRunDir():

    runDirLabel = 1
    runDirToLookFor = f"{originalDirectory}/magnetocosmicsRunDir_{runDirLabel}"

    while os.path.exists(runDirToLookFor):
        runDirLabel += 1
        runDirToLookFor = f"{originalDirectory}/magnetocosmicsRunDir_{runDirLabel}"

    return runDirLabel
    
runDirLabel = getUniqueLabelForRunDir()
magcosRunDir = f"{originalDirectory}/magnetocosmicsRunDir_{runDirLabel}"





#from numba import jit

import numpy as np
import pandas as pd
import tqdm
import pickle as pkl
import multiprocessing as mp
import warnings


def pandasSeriesInputExtension(numpyOperationFunction):

    def pandasSeriesFunction(pandasSeries):

        return numpyOperationFunction(Xpla=pandasSeries["Xpla"],
                                      Ypla=pandasSeries["Ypla"],
                                      Zpla=pandasSeries["Zpla"])

    return pandasSeriesFunction

def disableAngleLimiting(angleLimitFunction):

    def skipFunction(value):

        return value

    return skipFunction

@disableAngleLimiting
def sinArcSin(value):
    
    valueInRad = 2*np.pi*value/360
    
    outputValue = np.arcsin(np.sin(valueInRad))
    
    outputValueInDeg = outputValue * 360 / (2*np.pi)
    
    return outputValueInDeg

@disableAngleLimiting
def cosArcCos(value):
    
    valueInRad = 2*np.pi*value/360
    
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        outputValue = np.arccos(np.cos(valueInRad))
    
    outputValueInDeg = outputValue * 360 / (2*np.pi)
    
    return outputValueInDeg

#@jit(nopython=True)
def getZenithStringFromFilePath(filePath):
    
    zenithString = filePath.split("_")[1]
    
    return zenithString

#@jit(nopython=True)
def getAzimuthStringFromFilePath(filePath):
    
    azimuthString = (filePath.split("_")[2]).split(".out")[0]
    
    return azimuthString

@pandasSeriesInputExtension
#@jit(nopython=True)
def calculateTHETAplaOfRow(Xpla, Ypla, Zpla):
    
    #return np.arctan(np.sqrt(row["Xpla"]**2+row["Ypla"]**2)/row["Zpla"]) * 360/ (2* np.pi)
    return np.arctan(np.sqrt(Xpla**2+Ypla**2)/Zpla) * 360/ (2* np.pi)
    
@pandasSeriesInputExtension
#@jit(nopython=True)
def calculatePHIplaOfRow(Xpla, Ypla, Zpla):
    
    if Xpla > 0:
        
        outputValue = np.arctan(Ypla/Xpla)
        
    elif Xpla < 0:
            
        outputValue = np.arctan(Ypla/Xpla)+np.pi
    
    else:
        
        outputValue = np.pi/2
        
    return outputValue * 360/(2*np.pi)

def calculateAngleBetweenPosAndMom(row):
    
    degreesToRad = 2 * np.pi / 360
    
    thetaOne = row["Lat"] * degreesToRad
    thetaTwo = row["THETApla"] * degreesToRad
    phiOne = row["Long"] * degreesToRad
    phiTwo = row["PHIpla"] * degreesToRad
    
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        return finishPosMomAngleCalculation(thetaOne, thetaTwo, phiOne, phiTwo)

#@jit(nopython=True)
def finishPosMomAngleCalculation(thetaOne, thetaTwo, phiOne, phiTwo):
    return np.arccos((np.sin(thetaOne) * np.sin(thetaTwo) * np.cos(phiOne - phiTwo)) + 
            (np.cos(thetaOne) * np.cos(thetaTwo)))

def readOutputAngleFile(filePath):
    outputDF = pd.read_csv(filePath, 
            delimiter="\s+",
            index_col=False,
            skipfooter=1,
            header=None, 
            skiprows=1,
            engine="python")
            #engine="C")
    outputDF.columns = ["Rigidity","Filter","Lat", "Long", "Xpla", "Ypla", "Zpla"]
    
    #outputDF["Long"] = outputDF["Long"].apply(ifLessAdd)
    #outputDF["Long"] = outputDF["Long"].apply(sinArcSin)
    
    outputDF["Long"] = outputDF["Long"].apply(cosArcCos)
    outputDF["Lat"] = outputDF["Lat"].apply(sinArcSin)
    
    outputDF["initialZenith"] = getZenithStringFromFilePath(filePath)
    outputDF["initialAzimuth"] = getAzimuthStringFromFilePath(filePath)
    
    outputDF.zenith = getZenithStringFromFilePath(filePath)
    outputDF.azimuth = getAzimuthStringFromFilePath(filePath)
    
    outputDF["THETApla"] = outputDF.apply(lambda x:calculateTHETAplaOfRow(x),axis=1)
    #fullDF["PHIpla"] = np.piecewise(fullDF["Xpla"],
    #                                [fullDF["Xpla"] > 0, fullDF["Xpla"] < 0, fullDF["Xpla"] == 0.0],
    #                                [np.arctan(fullDF["Ypla"]/fullDF["Xpla"]),
    #                                 np.arctan(fullDF["Ypla"]/fullDF["Xpla"])+np.pi,
    #                                 np.pi/2])
    outputDF["PHIpla"] = outputDF.apply(lambda x:calculatePHIplaOfRow(x),axis=1)
    
    outputDF["angleDiffRad"] = outputDF.apply(lambda x:calculateAngleBetweenPosAndMom(x),axis=1)
    
    outputDF["weightingFactor"] = np.sin(2 * outputDF["angleDiffRad"])
    
    return outputDF[outputDF["Filter"] == 1]

def getPosAndMomDirFromFilename(filePath):

    latitude = float(filePath.split("POS_")[1].split("_")[0])
    longitude = float(filePath.split("POS_")[1].split("_")[1])
    zenith = float(filePath.split("MOM_")[1].split("_")[0])
    azimuth = float(filePath.split("MOM_")[1].split("_")[1].split(".")[0])

    return latitude, longitude, zenith, azimuth

def read_cutoff_values_from_file(filePath):

    with open(filePath, 'r') as f:
        lines = f.read().splitlines()
        last_line = lines[-1]

    splitLastLine = last_line.split()

    rigidityDict = {"Rl":float(splitLastLine[1]),
                    "Rc":float(splitLastLine[3]),
                    "Ru":float(splitLastLine[5]),}

    return rigidityDict

def readNewOutputAngleFile(filePath):

    #print(filePath)
    
    outputDF = pd.read_csv(filePath, 
            delimiter="\s+",
            index_col=False,
            skipfooter=1,
            header=None, 
            skiprows=1,
            engine="python")
            #engine="C")
    
    outputDF.columns = ["Rigidity","Filter","Lat", "Long", "Xpla", "Ypla", "Zpla"]
    
    outputDF["Long"] = outputDF["Long"].apply(cosArcCos)
    outputDF["Lat"] = outputDF["Lat"].apply(sinArcSin)

    outputDF.latitude, outputDF.longitude, outputDF.zenith, outputDF.azimuth = getPosAndMomDirFromFilename(filePath)
    outputDF["initialLatitude"] = outputDF.latitude
    outputDF["initialLongitude"] = outputDF.longitude
    outputDF["initialZenith"] = outputDF.zenith
    outputDF["initialAzimuth"] = outputDF.azimuth
    
    outputDF["THETApla"] = outputDF.apply(lambda x:calculateTHETAplaOfRow(x),axis=1)
    outputDF["PHIpla"] = outputDF.apply(lambda x:calculatePHIplaOfRow(x),axis=1)
    
    outputDF["angleDiffRad"] = outputDF.apply(lambda x:calculateAngleBetweenPosAndMom(x),axis=1)
    
    outputDF["weightingFactor"] = np.sin(2 * outputDF["angleDiffRad"])

    rigidity_cutoff_dict = read_cutoff_values_from_file(filePath)

    outputDF["Rlower"] = rigidity_cutoff_dict["Rl"]
    outputDF["Reffective"] = rigidity_cutoff_dict["Rc"]
    outputDF["Rupper"] = rigidity_cutoff_dict["Ru"]
    
    return outputDF

#@mem.cache
def readLotsOfOutputAngleFiles(listOfFilePaths):
    listOfAllDFs = [readNewOutputAngleFile(filePath) for filePath in listOfFilePaths]
    #with mp.Pool(processes=4) as mpPool:
    #    listOfAllDFs = mpPool.map(pd.read_csv,listOfFilePaths)

    return pd.concat(listOfAllDFs)

#@mem.cache
def readLotsOfOutputAngleFilesChunked(listOfFilePaths,nChunks=100, save_as_pkl=False):

    chunkedListOfFilePaths = np.array_split(listOfFilePaths,
                                            min(nChunks,len(listOfFilePaths)))

    listOfAllDFs = [readLotsOfOutputAngleFiles(filePath) for filePath in tqdm.tqdm(chunkedListOfFilePaths) if len(filePath) != 0]
    with mp.Pool(processes=5) as mpPool:
        listOfAllDFs = mpPool.map(readLotsOfOutputAngleFiles,tqdm.tqdm(chunkedListOfFilePaths))

    fullDF = pd.concat(listOfAllDFs)

    if save_as_pkl == True:
        with open("savedFullDFlist.pkl", "wb") as f:
            pkl.dump(fullDF,f)

    return fullDF


import time
import numpy as np
import datetime as dt

from AsympDirsCalculator.AsympDirsTools import get_magcos_asymp_dirs, get_magcos_vcutoffs

def test_simple_asymp_dir_run():

    outputtedAsympDirs = get_magcos_asymp_dirs(array_of_lats_and_longs = np.array([[0.0, 0.0],[1.0,1.0],[50.0,0.0]]))

    print(outputtedAsympDirs)

    pass

def test_simple_asymp_dir_run_verbose():

    outputtedAsympDirs = get_magcos_asymp_dirs(array_of_lats_and_longs = np.array([[0.0, 0.0],[1.0,1.0],[50.0,0.0]]), full_output=True)

    print(outputtedAsympDirs)

    pass

def test_simple_vcutoff_run():

    outputted_vcutoffs = get_magcos_vcutoffs(array_of_lats_and_longs = np.array([[0.0, 0.0],
                                                                                 [1.0,1.0],
                                                                                 [50.0,0.0]]))

    print(outputted_vcutoffs)

    pass

def test_simple_asymp_dir_run_cache():

    datetimeToUse = dt.datetime(year=2010, month=10, day=27)
    outputtedAsympDirs = get_magcos_asymp_dirs(array_of_lats_and_longs = np.array([[0.0, 0.0],[1.0,1.0]]),
                                               dateAndTime=datetimeToUse,
                                               cache=True)

    print(outputtedAsympDirs)

    pass

def test_full_world_run():

    start = time.time()
    get_magcos_asymp_dirs()
    end = time.time()
    print(end - start)
    
    with open("previousWorldRunTime.txt","w") as runTimeFile:
        runTimeFile.write(f"{end - start}")

def test_full_world_run_cache():

    datetimeToUse = dt.datetime(year=2010, month=10, day=27)

    start = time.time()
    asympDirList = get_magcos_asymp_dirs(KpIndex=6, dateAndTime=datetimeToUse, cache=True)
    end = time.time()
    print(end - start)


from setuptools import find_packages, setup
import os
import glob as gb

# get requirements for installation
lib_folder = os.path.dirname(os.path.realpath(__file__))
requirement_path = lib_folder + '/requirements.txt'
install_requires = [] # Here we'll get: ["gunicorn", "docutils>=0.3", "lxml==0.5a7"]
if os.path.isfile(requirement_path):
    with open(requirement_path) as f:
        install_requires = f.read().splitlines()

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='AsympDirsCalculator',
    packages=find_packages(exclude='tests'),
    package_data={"AsympDirsCalculator":["magcos_running_scripts/runNoRewriteMAGCOSsimulation.sh",
                                         "magcos_running_scripts/AsymptoticDirection.g4mac",
                                         ]},
    version='1.0.12',
    description='Python library containing tools for calculating asymptotic directions and vertical cut-off rigidities.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Space Environment and Protection Group, University of Surrey',
    url='https://github.com/ssc-maire/AsymptoticDirectionsCalculator-public',
    keywords = 'space physics earth asymptotic trajectory geomagnetic rigidity magnetocosmics',
    license='GNU General Public License v3.0',
    install_requires= ['numpy>=1.21.6',
                        'pandas>=1.3.5',
                        'setuptools>=45.2.0',
                        'tqdm>=4.30.0',
                        'joblib>=1.2.0',
                        'wheel>=0.38.4'],
    #install_requires,
    setup_requires=['pytest-runner','wheel'],
    tests_require=['pytest'],
)



#!/bin/python3

import datetime as dt
import numpy as np
import pandas as pd

from .MAGNETOCOSMICSrun import MAGNETOCOSMICSrun, get_magcos_cache_function


class MAGNETOCOSMICSrunManager():

    _runMode = "General"

    def __init__(self, 
                 dateAndTime:dt.datetime,
                 KpIndex:int,
                 cache=False,
                 ):

        self._date_and_time = dateAndTime
        self._kp_index = KpIndex
        self.set_MAGCOS_run_function(cache=cache)

    def set_rigidity_run_params(self, highestMaxRigValue, maxRigValue, minRigValue, nIncrements_high, nIncrements_low):

        self._highestMaxRigValue = highestMaxRigValue
        self._maxRigValue = maxRigValue
        self._minRigValue = minRigValue
        self._nIncrements_high = nIncrements_high
        self._nIncrements_low = nIncrements_low

    def set_particle_direction_params(self, array_of_zeniths_and_azimuths):

        self._array_of_zeniths_and_azimuths = array_of_zeniths_and_azimuths

    def setMAGNETOCOSMICSdatetimeParameters(self,dateAndTime:dt.datetime,KpIndex:int):
        self._dateAndTime = dateAndTime
        self._KpIndex = KpIndex

    def setRunMode(self, runMode):
        self._runMode = runMode

    def set_MAGCOS_run_function(self, cache:bool):

        if cache == True:
            self.MAGCOS_run_callable = get_magcos_cache_function()
        else:
            self.MAGCOS_run_callable = MAGNETOCOSMICSrun

    def run(self, array_of_lats_and_longs:np.array):

        lowRigRun = self.MAGCOS_run_callable(array_of_lats_and_longs, self._array_of_zeniths_and_azimuths,
                                         self._date_and_time, self._kp_index,
                                         self._maxRigValue,self._minRigValue,self._nIncrements_low)

        highRigRun = self.MAGCOS_run_callable(array_of_lats_and_longs, self._array_of_zeniths_and_azimuths,
                                         self._date_and_time, self._kp_index,
                                         self._highestMaxRigValue,self._maxRigValue,self._nIncrements_high)

        highRigRun.full_rigidity_DF["Rlower"] = float(lowRigRun.full_rigidity_DF["Rlower"].iloc[0])
        highRigRun.full_rigidity_DF["Reffective"] = float(lowRigRun.full_rigidity_DF["Reffective"].iloc[0])
        highRigRun.full_rigidity_DF["Rupper"] = float(lowRigRun.full_rigidity_DF["Rupper"].iloc[0])
        
        self.fullRigidityDF = pd.concat([highRigRun.full_rigidity_DF,
                                         lowRigRun.full_rigidity_DF],ignore_index=True)

    def getOutputRigidities(self):
        return self.fullRigidityDF

if __name__ == "__main__":
    testMAGCOSrunManager = MAGNETOCOSMICSrunManager()
    testMAGCOSrunManager.run()

    print(testMAGCOSrunManager.getOutputRigidities())